disp(' ')
disp(' srs_tripartite.m   ver 1.2   March 27, 2010')
disp(' by Tom Irvine   Email: tomirvine@aol.com')
disp(' ')
disp(' This program calculates the shock response spectrum')
disp(' of an acceleration time history, which is pre-loaded into Matlab.')
disp(' The time history must have two columns: time(sec) & acceleration')
disp(' ')
%
clear t;
clear y;
clear yy;
clear n;
clear fn;
clear a1;
clear a2
clear b1;
clear b2;
clear jnum;
clear THM;
clear resp;
clear x_pos;
clear x_neg;
%
iunit=input(' Enter acceleration unit:   1= G   2= m/sec^2  ');	
%
disp(' ')
disp(' Select file input method ');
disp('   1=external ASCII file ');
disp('   2=file preloaded into Matlab ');
disp('   3=Excel file ');
file_choice = input('');
%
if(file_choice==1)
        [filename, pathname] = uigetfile('*.*');
        filename = fullfile(pathname, filename);
        fid = fopen(filename,'r');
        THM = fscanf(fid,'%g %g',[2 inf]);
        THM=THM';
end
if(file_choice==2)
        THM = input(' Enter the matrix name:  ');
end
if(file_choice==3)
        [filename, pathname] = uigetfile('*.*');
        xfile = fullfile(pathname, filename);
%        
        THM = xlsread(xfile);
%         
end
%
t=double(THM(:,1));
y=double(THM(:,2));
%
tmx=max(t);
tmi=min(t);
n = length(y);
%
out1 = sprintf('\n  %d samples ',n);
disp(out1)
%
dt=(tmx-tmi)/(n-1);
sr=1./dt;
%
disp(' ')
disp(' Time Step ');
dtmin=min(diff(t));
dtmax=max(diff(t));
%
out4 = sprintf(' dtmin  = %8.4g sec  ',dtmin);
out5 = sprintf(' dt     = %8.4g sec  ',dt);
out6 = sprintf(' dtmax  = %8.4g sec  ',dtmax);
disp(out4)
disp(out5)
disp(out6)
%
disp(' ')
disp(' Sample Rate ');
out4 = sprintf(' srmin  = %8.4g samples/sec  ',1/dtmax);
out5 = sprintf(' sr     = %8.4g samples/sec  ',1/dt);
out6 = sprintf(' srmax  = %8.4g samples/sec  \n',1/dtmin);
disp(out4)
disp(out5)
disp(out6)
%
ncontinue=1;
if(((dtmax-dtmin)/dt)>0.01)
    disp(' ')
    disp(' Warning:  time step is not constant.  Continue calculation? 1=yes 2=no ')
    ncontinue=input(' ')
end
if(ncontinue==1)
%
    fn(1)=input(' Enter the starting frequency (Hz)  ');
    if fn(1)>sr/30.
        fn(1)=sr/30.;
    end
%
    idamp=input(' Enter damping format:  1= damping ratio   2= Q  ');	
%
    disp(' ')
    if(idamp==1)
        damp=input(' Enter damping ratio (typically 0.05) ');
    else
        Q=input(' Enter the amplification factor (typically Q=10) ');
        damp=1./(2.*Q);
    end
%
    disp(' ')
    disp(' Select algorithm: ')
    disp(' 1=Kelly-Richman  2=Smallwood ');
    ialgorithm=input(' ');
%
    disp(' ');
    disp(' Include residual? ');
    disp('  1=yes  2=no ')
    ire=input(' ');
%
    if(ire==1)
%
%   Add trailing zeros for residual response
%
       tmax=(tmx-tmi) + 1./fn(1);
    else
       tmax=(tmx-tmi);
    end
%
    limit = round( tmax/dt );
    n=limit;
    yy=zeros(1,limit);
    for i=1:length(y)
        yy(i)=y(i);
    end    
%
    disp(' ')
    disp(' Calculating response..... ')
%
%  SRS engine
%
    for j=1:1000
%
        omega=2.*pi*fn(j);
        omegad=omega*sqrt(1.-(damp^2));
        cosd=cos(omegad*dt);
        sind=sin(omegad*dt);
        domegadt=damp*omega*dt;
%
        if(ialgorithm==1)
            a1(j)=2.*exp(-domegadt)*cosd;
            a2(j)=-exp(-2.*domegadt);
            b1(j)=2.*domegadt;
            b2(j)=omega*dt*exp(-domegadt);
            b2(j)=b2(j)*( (omega/omegad)*(1.-2.*(damp^2))*sind -2.*damp*cosd );
            b3(j)=0;
%
        else
	        E=exp(-damp*omega*dt);
		    K=omegad*dt;
		    C=E*cos(K);
		    S=E*sin(K);
		    Sp=S/K;
%
	        a1(j)=2*C;
		    a2(j)=-E^2;
		    b1(j)=1.-Sp;
		    b2(j)=2.*(Sp-C);
		    b3(j)=E^2-Sp;
        end
        forward=[ b1(j),  b2(j),  b3(j) ];    
        back   =[     1, -a1(j), -a2(j) ];    
%    
        resp=filter(forward,back,yy);
%
        x_pos(j)= max(resp);
        x_neg(j)= min(resp);
%   
        jnum=j; 
        if  fn(j) > sr/8.
            break
        end
        fn(j+1)=fn(1)*(2. ^ (j*(1./12.)));    
    end
%
    maximaxSRS=max(x_pos,abs(x_neg)); 
%
    fmax=0;
    zmax=0;
    for(i=1:length(fn))
        if(x_pos(i)>zmax)
            zmax=x_pos(i);
            fmax=fn(i);
        end
        if(abs(x_neg(i))>zmax)
            zmax=abs(x_neg(i));
            fmax=fn(i);
        end 
    end
%
    fmax=max(fn);
    fmin=fmax/10.;
%
    fmax= 10^(round(log10(fmax)+0.5));
%
    if  fn(1) >= 0.1
        fmin=0.1;
    end
    if  fn(1) >= 1
        fmin=1;
    end
    if  fn(1) >= 10
        fmin=10;
    end
    if  fn(1) >= 100
        fmin=100;
    end
%
    if(iunit==1)
        out5 = sprintf('\n Absolute Peak is %10.5g G at %10.5g Hz ',zmax,fmax);
    else
        out5 = sprintf('\n Absolute Peak is %10.5g m/sec^2 at %10.5g Hz ',zmax,fmax);
    end
    disp(out5)
%
%  Output options
%
%  Plot SRS
%
    SRS_pn=[fn',x_pos',abs(x_neg)'];
    SRS_max=[fn',x_pos',maximaxSRS']; 
%
    disp(' Matlab matrices: ')
    disp('      SRS_pn  - Acceleration SRS positive & negative ')
    disp('      SRS_max - Acceleration SRS maximax             ') 
%
    disp(' ')
    vchoice=1;
    if(vchoice==1)
        disp(' select plot type:  1=positive & negative   2=maximax ');
        ipt=input(' ');     
%
%   Convert to pseudo velocity
%
        for j=1:jnum  
            if iunit==1   
                x_pos(j)=386.*x_pos(j)/(2.*pi*fn(j));
                x_neg(j)=386.*x_neg(j)/(2.*pi*fn(j));   
            else
                x_pos(j)=x_pos(j)/(2.*pi*fn(j));
                x_neg(j)=x_neg(j)/(2.*pi*fn(j));   
            end
        end    
%
        srs_max = max(x_pos);
        if( max( abs(x_neg) ) > srs_max & fn(j) >= fmin & fn(j)<=fmax)
            srs_max = max( abs(x_neg ));
        end
        srs_min = min(x_pos);
        if( min( abs(x_neg) ) < srs_min & fn(j) >= fmin & fn(j)<=fmax)
            srs_min = min( abs(x_neg ));
        end  
%
        clear xpn;
        xpn=max(x_pos,abs(x_neg)); 
        figure(2);
        if(ipt==1)
            plot(fn,x_pos,fn,abs(x_neg),'-.','LineWidth',2);
            legend ('positive','negative',2);   
        else     
            plot(fn,xpn,'LineWidth',2);    
        end
        pseudo_velocity_pn=[fn',x_pos',abs(x_neg)'];
        pseudo_velocity_max=[fn',xpn'];  
%
        if iunit==1
            ylabel('Velocity (in/sec)');
            scale=386.;
        else
            ylabel('Velocity (m/sec)');
            scale=1.;
        end
        xlabel('Natural Frequency (Hz)');
        Q=1./(2.*damp);
        out5 = sprintf(' Pseudo Velocity Shock Response Spectrum Q=%g ',Q);
%%%%%        title(out5);
        grid;
        set(gca,'MinorGridLineStyle',':','GridLineStyle',':','XScale','log','YScale','log');
%
        ymax= 10^(round(log10(srs_max)+0.6));
        ymin= 10^(round(log10(srs_min)-0.6));
%
        axis([fmin,fmax,ymin,ymax]);
        hold on;
%        v=a/(2pif)
        f1=fmin;
        v2=ymin;
%
%    acceleration lines
%
        for(jk=-5:5)
            for(k=1:10)
                amp=k*10^(jk);
                v1=amp*scale/(2*pi*fmin);
                f2=amp*scale/(2*pi*v2);
                a=[f1 v1; f2 v2];
                plot(a(:,1),a(:,2),'color','k','LineWidth',0.25);
                hold on;   
            end
        end  
% 
%    relative displacement lines
%
        for(jk=-8:5)
            for(k=1:10)
                amp=k*10^(jk);
                v1=amp*(2*pi*fmin);
                v2=amp*(2*pi*fmax);  
                f2=fmax;
                a=[f1 v1; f2 v2];
                plot(a(:,1),a(:,2),'color','k','LineWidth',0.25);
                hold on;   
            end
        end  
 %     
    end
    theta=-45;
%    
    nxd=round(log10(fmax/fmin));
    nyd=round(log10(ymax/ymin));
%       
    if(nxd==nyd)
        theta=-39;
    end
    if(nxd==2 & nyd==3)
        theta=-29;
    end   
    if(nxd==3 & nyd==2)
        theta=-48.5;
    end   
     if(nxd==3 & nyd==4)
        theta=-34;
    end      
    if(nxd==4 & nyd==3)
        theta=-46;
    end    
    set(gca,'DefaultTextRotation',theta);
%    
    amax=ymax*(2*pi*fmin)/scale;
    amin=ymin*(2*pi*fmin)/scale;
%   
    if(iunit==1)
        if(amin<=0.01 & amax>=0.01)
            text('Position',[fmin  0.012*scale/(2*pi*fmin)],'String','0.01 G','color','k','FontWeight','bold','FontSize',13)  
        end
        if(amin<=0.1 & amax>=0.1)   
            text('Position',[fmin   0.12*scale/(2*pi*fmin)],'String', '0.1 G','color','k','FontWeight','bold','FontSize',13)     
        end   
        if(amin<=1 & amax>=1)
            text('Position',[fmin    1.2*scale/(2*pi*fmin)],'String',   '1 G','color','k','FontWeight','bold','FontSize',13)        
        end   
        if(amin<=10 & amax>=10) 
            text('Position',[fmin     12*scale/(2*pi*fmin)],'String',  '10 G','color','k','FontWeight','bold','FontSize',13)        
        end    
        if(amin<=100 & amax>=100)
            text('Position',[fmin    120*scale/(2*pi*fmin)],'String', '100 G','color','k','FontWeight','bold','FontSize',13)        
        end    
        if(amin<=1000 & amax>=1000) 
            text('Position',[fmin   1200*scale/(2*pi*fmin)],'String',  '1K G','color','k','FontWeight','bold','FontSize',13)      
        end   
        if(amin<=10000 & amax>=10000)   
            text('Position',[fmin  12000*scale/(2*pi*fmin)],'String', '10K G','color','k','FontWeight','bold','FontSize',13)   
        end   
        if(amin<=100000 & amax>=100000)
            text('Position',[fmin 120000*scale/(2*pi*fmin)],'String','100K G','color','k','FontWeight','bold','FontSize',13)
        end 
    else
         if(amin<=0.01 & amax>=0.01)
            text('Position',[fmin  0.012*scale/(2*pi*fmin)],'String','0.01 m/s^2','color','k','FontWeight','bold','FontSize',12)  
        end
        if(amin<=0.1 & amax>=0.1)   
            text('Position',[fmin   0.12*scale/(2*pi*fmin)],'String', '0.1 m/s^2','color','k','FontWeight','bold','FontSize',12)     
        end   
        if(amin<=1 & amax>=1)
            text('Position',[fmin    1.2*scale/(2*pi*fmin)],'String',   '1 m/s^2','color','k','FontWeight','bold','FontSize',12)        
        end   
        if(amin<=10 & amax>=10) 
            text('Position',[fmin     12*scale/(2*pi*fmin)],'String',  '10 m/s^2','color','k','FontWeight','bold','FontSize',12)        
        end    
        if(amin<=100 & amax>=100)
            text('Position',[fmin    120*scale/(2*pi*fmin)],'String', '100 m/s^2','color','k','FontWeight','bold','FontSize',12)        
        end    
        if(amin<=1000 & amax>=1000) 
            text('Position',[fmin   1200*scale/(2*pi*fmin)],'String',  '1K m/s^2','color','k','FontWeight','bold','FontSize',12)      
        end   
        if(amin<=10000 & amax>=10000)   
            text('Position',[fmin  12000*scale/(2*pi*fmin)],'String', '10K m/s^2','color','k','FontWeight','bold','FontSize',12)   
        end   
        if(amin<=100000 & amax>=100000)
            text('Position',[fmin 120000*scale/(2*pi*fmin)],'String','100K m/s^2','color','k','FontWeight','bold','FontSize',12)
        end        
    end
%    
%%%%%  v=a/(2pif)
%
    aamax=ymax*(2*pi*fmax)/scale;
%
    if(iunit==1)
        if(amax<=0.01 & 0.01<aamax)
            text('Position',[0.012*scale/(2*pi*ymax) ymax],'String','0.01 G','color','k','FontWeight','bold','FontSize',13)  
        end
        if(amax<=0.1 & 0.1<aamax)
            text('Position',[0.12*scale/(2*pi*ymax) ymax],'String','0.1 G','color','k','FontWeight','bold','FontSize',13)  
        end
        if(amax<=1 & 1<aamax)
            text('Position',[1.2*scale/(2*pi*ymax) ymax],'String','1 G','color','k','FontWeight','bold','FontSize',13)  
        end
        if(amax<=10 & 10<aamax)
            text('Position',[12*scale/(2*pi*ymax) ymax],'String','10 G','color','k','FontWeight','bold','FontSize',13)  
        end
        if(amax<=100 & 100<aamax)
            text('Position',[120*scale/(2*pi*ymax) ymax],'String','100 G','color','k','FontWeight','bold','FontSize',13)  
        end   
        if(amax<=1000 & 1000<aamax)
            text('Position',[1200*scale/(2*pi*ymax) ymax],'String','1K G','color','k','FontWeight','bold','FontSize',13)  
        end
        if(amax<=10000 & 10000<aamax)
            text('Position',[12000*scale/(2*pi*ymax) ymax],'String','10K G','color','k','FontWeight','bold','FontSize',13)  
        end
        if(amax<=100000 & 100000<aamax)
            text('Position',[120000*scale/(2*pi*ymax) ymax],'String','100K G','color','k','FontWeight','bold','FontSize',13)  
        end    
    else
         if(amax<=0.01 & 0.01<aamax)
            text('Position',[0.012*scale/(2*pi*ymax) ymax],'String','0.01 m/s^2','color','k','FontWeight','bold','FontSize',12)  
        end
        if(amax<=0.1 & 0.1<aamax)
            text('Position',[0.12*scale/(2*pi*ymax) ymax],'String','0.1 m/s^2','color','k','FontWeight','bold','FontSize',12)  
        end
        if(amax<=1 & 1<aamax)
            text('Position',[1.2*scale/(2*pi*ymax) ymax],'String','1 m/s^2','color','k','FontWeight','bold','FontSize',12)  
        end
        if(amax<=10 & 10<aamax)
            text('Position',[12*scale/(2*pi*ymax) ymax],'String','10 m/s^2','color','k','FontWeight','bold','FontSize',12)  
        end
        if(amax<=100 & 100<aamax)
            text('Position',[120*scale/(2*pi*ymax) ymax],'String','100 m/s^2','color','k','FontWeight','bold','FontSize',12)  
        end   
        if(amax<=1000 & 1000<aamax)
            text('Position',[1200*scale/(2*pi*ymax) ymax],'String','1K m/s^2','color','k','FontWeight','bold','FontSize',12)  
        end
        if(amax<=10000 & 10000<aamax)
            text('Position',[12000*scale/(2*pi*ymax) ymax],'String','10K m/s^2','color','k','FontWeight','bold','FontSize',12)  
        end
        if(amax<=100000 & 100000<aamax)
            text('Position',[120000*scale/(2*pi*ymax) ymax],'String','100K m/s^2','color','k','FontWeight','bold','FontSize',12)  
        end         
    end
%%%%
    set(gca,'DefaultTextRotation',-theta);
    dmax=ymax/(2*pi*fmax);
    dmin=ymin/(2*pi*fmax);
%  
    if(iunit==1)
        if(dmin<=10 & dmax>=10)
            text('Position',[0.32*fmax  4*(2*pi*fmax)],'String','10 in','color','k','FontWeight','bold','FontSize',13)  
        end
        if(dmin<=1 & dmax>=1)
            text('Position',[0.32*fmax  0.4*(2*pi*fmax)],'String','1 in','color','k','FontWeight','bold','FontSize',13)  
        end
        if(dmin<=0.1 & dmax>=0.1)
            text('Position',[0.32*fmax  0.04*(2*pi*fmax)],'String','0.1 in','color','k','FontWeight','bold','FontSize',13)  
        end
        if(dmin<=0.01 & dmax>=0.01)
            text('Position',[0.32*fmax  0.004*(2*pi*fmax)],'String','0.01 in','color','k','FontWeight','bold','FontSize',13)  
        end
        if(dmin<=0.001 & dmax>=0.001)
            text('Position',[0.32*fmax  0.0004*(2*pi*fmax)],'String','0.001 in','color','k','FontWeight','bold','FontSize',13)  
        end
        if(dmin<=0.0001 & dmax>=0.0001)
            text('Position',[0.32*fmax  0.00004*(2*pi*fmax)],'String','1e-04 in','color','k','FontWeight','bold','FontSize',13)  
        end      
    else
        if(dmin<=10 & dmax>=10)
            text('Position',[0.32*fmax  4*(2*pi*fmax)],'String','10 m','color','k','FontWeight','bold','FontSize',12)  
        end
        if(dmin<=1 & dmax>=1)
            text('Position',[0.32*fmax  0.4*(2*pi*fmax)],'String','1 m','color','k','FontWeight','bold','FontSize',12)  
        end
        if(dmin<=0.1 & dmax>=0.1)
            text('Position',[0.32*fmax  0.04*(2*pi*fmax)],'String','0.1 m','color','k','FontWeight','bold','FontSize',12)  
        end
        if(dmin<=0.01 & dmax>=0.01)
            text('Position',[0.32*fmax  0.004*(2*pi*fmax)],'String','10 cm','color','k','FontWeight','bold','FontSize',12)  
        end
        if(dmin<=0.001 & dmax>=0.001)
            text('Position',[0.32*fmax  0.0004*(2*pi*fmax)],'String','1 cm','color','k','FontWeight','bold','FontSize',12)  
        end
        if(dmin<=0.0001 & dmax>=0.0001)
            text('Position',[0.32*fmax  0.00004*(2*pi*fmax)],'String','10 mm','color','k','FontWeight','bold','FontSize',12)  
        end  
        if(dmin<=0.00001 & dmax>=0.00001)
            text('Position',[0.32*fmax  0.000004*(2*pi*fmax)],'String','1 mm','color','k','FontWeight','bold','FontSize',12)  
        end 
        if(dmin<=0.000001 & dmax>=0.000001)
            text('Position',[0.32*fmax  0.0000004*(2*pi*fmax)],'String','0.1 mm','color','k','FontWeight','bold','FontSize',12)  
        end          
    end
%%%%
    ddmax=ymax/(2*pi*fmin);
    ddmin=ymax/(2*pi*fmax); 
%  
    if(iunit==1)
%        
        if(ddmin<=0.01 & 0.01<ddmax)
            text('Position',[ymax/(2*pi*0.01) ymax],'String','0.01 in','color','k','FontWeight','bold','FontSize',13)  
        end
 %
        if(ddmin<=0.1 & 0.1<ddmax)
            text('Position',[ymax/(2*pi*0.1) ymax],'String','0.1 in','color','k','FontWeight','bold','FontSize',13)  
        end
 %
        if(ddmin<=1 & 1<ddmax)
            text('Position',[ymax/(2*pi*1) ymax],'String','1 in','color','k','FontWeight','bold','FontSize',13)  
        end
  %
        if(ddmin<=10 & 10<ddmax)
            text('Position',[ymax/(2*pi*10) ymax],'String','10 in','color','k','FontWeight','bold','FontSize',13)  
        end 
 %
        if(ddmin<=100 & 100<ddmax)
            text('Position',[ymax/(2*pi*100) ymax],'String','100 in','color','k','FontWeight','bold','FontSize',13)  
        end
    else
      %        
        if(ddmin<=0.000001 & 0.000001<ddmax)
            text('Position',[ymax/(2*pi*0.000001) ymax],'String','0.1 mm','color','k','FontWeight','bold','FontSize',12)  
        end           
    %        
        if(ddmin<=0.00001 & 0.00001<ddmax)
            text('Position',[ymax/(2*pi*0.00001) ymax],'String','1 mm','color','k','FontWeight','bold','FontSize',12)  
        end         
 %        
        if(ddmin<=0.0001 & 0.0001<ddmax)
            text('Position',[ymax/(2*pi*0.0001) ymax],'String','10 mm','color','k','FontWeight','bold','FontSize',12)  
        end          
%        
        if(ddmin<=0.001 & 0.001<ddmax)
            text('Position',[ymax/(2*pi*0.001) ymax],'String','1 cm','color','k','FontWeight','bold','FontSize',12)  
        end        
%        
        if(ddmin<=0.01 & 0.01<ddmax)
            text('Position',[ymax/(2*pi*0.01) ymax],'String','10 cm','color','k','FontWeight','bold','FontSize',12)  
        end
 %
        if(ddmin<=0.1 & 0.1<ddmax)
            text('Position',[ymax/(2*pi*0.1) ymax],'String','0.1 m','color','k','FontWeight','bold','FontSize',12)  
        end
 %
        if(ddmin<=1 & 1<ddmax)
            text('Position',[ymax/(2*pi*1) ymax],'String','1 m','color','k','FontWeight','bold','FontSize',12)  
        end
  %
        if(ddmin<=10 & 10<ddmax)
            text('Position',[ymax/(2*pi*10) ymax],'String','10 m','color','k','FontWeight','bold','FontSize',12)  
        end 
 %
        if(ddmin<=100 & 100<ddmax)
            text('Position',[ymax/(2*pi*100) ymax],'String','100 m','color','k','FontWeight','bold','FontSize',12)  
        end        
    end
%%%%
    grid on;     
%
    disp(' Matlab matrices: ')
    disp('      pseudo_velocity_pn  - pseudo_velocity SRS positive & negative ')
    disp('      pseudo_velocity_max - pseudo_velocity SRS maximax             ') 
%
end
hold off;